@@ -12,6 +12,7 @@ import json
 from urllib.parse import urlparse
 from urllib.request import urlretrieve
 import tarfile
+from scipy.spatial.distance import pdist,squareform
 from nufeb_tools import __version__
 
 urls= ['https://github.com/Jsakkos/nufeb-tools/raw/main/data/runs.tar']
@@ -40,7 +41,7 @@ class get_data:
     def __init__(self,directory=None,id=None,test=None,timestep=10):
         self.timestep=timestep
         if test:
-            self.directory = str((Path.home()) / '.nufeb_tools' / 'data' / 'Run_26_90_83_1')
+            self.directory = str((Path.home()) / '.nufeb_tools' / 'data' / 'Run_60_18_63_1_2021-06-23')
             if not os.path.isdir(self.directory):
                 download_test_data()     
             self.get_local_data()
@@ -64,6 +65,7 @@ class get_data:
             self.numsteps = len(self.timepoints)
         except AttributeError:
             print('Missing HDF5 file')
+        self.collect_positions()
         
     def get_local_data(self):
         """
@@ -199,13 +201,21 @@ class get_data:
         
         Returns:
             df (pandas.DataFrame):
-                Dataframe containing ID, type,x, y, z columns
+                Dataframe containing Timestep, ID, type,x, y, z columns
         """
-        return pd.concat([pd.Series(self.h5['id'][str(timepoint)],name='ID'),
+        return pd.concat([pd.Series(np.ones(self.h5['x'][str(timepoint)].len())*int(timepoint),dtype=int,name='Timestep'),
+        pd.Series(self.h5['id'][str(timepoint)],name='ID'),
         pd.Series(self.h5['type'][str(timepoint)],name='type'),
         pd.Series(self.h5['x'][str(timepoint)],name='x'),
         pd.Series(self.h5['y'][str(timepoint)],name='y'),
         pd.Series(self.h5['z'][str(timepoint)],name='z')],axis=1)
+    def collect_positions(self):
+        """get cell locations for all timesteps and aggregate into a dataframe
+        """
+        dfs = list()
+        for t in self.timepoints:
+            dfs.append(self.get_positions(t))
+        self.positions = pd.concat(dfs)
     def get_neighbor_distance(self,id,timepoint):
         """
         Get the nearest neighbor cell distances
@@ -216,13 +226,37 @@ class get_data:
             timepoint (int):
                 The timepoint to check the neighbor distances from
         Returns:
-            df (pandas.DataFrame):
+            (pandas.DataFrame):
                 Dataframe containing ID, type, Distance
         """
-        df = self.get_positions(timepoint)
+        # TODO Speed up or parallelize this computation
+        df = self.positions.set_index(['Timepoint','ID'])
         temp = (df[df.ID ==id][['x','y','z']].squeeze() - df[df.ID !=id][['x','y','z']])**2
         dist = pd.Series(np.sqrt(temp.x + temp.y + temp.z),name='Distance')
         return pd.concat([df[df.ID !=id][['ID','type']],dist],axis=1).reset_index(drop=True)
+    def get_neighbors(self,timestep):
+        """
+        Get the nearest neighbor cell distances
+
+        Args:
+            timestep (int): 
+                The timepoint to check the neighbor distances from
+
+        Returns:
+            pd.DataFrame:
+                Pandas dataframe containing pairwise neighbor distances
+        """
+        df = self.positions
+        df2 = df[df.Timestep == timestep].set_index(['ID'])
+        df2.sort_index(inplace=True)
+        # distances =pdist(df2[['x','y','z']])
+        pairwise = pd.DataFrame(
+            squareform(pdist(df2[['x','y','z']])),
+            columns = df2.index,
+            index = df2.index
+        )
+        pairwise[pairwise ==0] = np.nan
+        return pairwise
 
     def get_local_con(self,nutrient,timestep,cellID):
         """
@@ -262,6 +296,7 @@ class get_data:
             fitness (float):
                 The Monod growth rate (1/s)
         """
+        # TODO Speed up or parallelize this computation
         if self.h5['type'].__contains__(str(timestep)): 
             cell_type = self.h5['type'][str(timestep)][np.where(self.h5['id'][str(timestep)].__array__() == cellID)[0][0]]
         else:
@@ -324,7 +359,7 @@ def download_test_data(urls=urls):
     cp_dir.mkdir(exist_ok=True)
     data_dir = cp_dir.joinpath('data')
     data_dir.mkdir(exist_ok=True)
-
+    # TODO Add progress bar
     for url in urls:
         parts = urlparse(url)
         filename = os.path.basename(parts.path)
@@ -355,6 +390,7 @@ def upload_datafed(file, title, collection_id,metadata_file):
     filename = file
     file_title= title
     global_coll_id = collection_id
+    from datafed.CommandLib import API
     df_api = API()
     if metadata_file != '':
         pkl_file = metadata_file
